@@ -10,7 +10,6 @@ The View:
 """
 import collections
 import typing
-import os
 
 import blinker
 import sortedcontainers
@@ -104,7 +103,7 @@ orders = [
 ]
 
 
-class View(collections.Sequence):
+class View(collections.abc.Sequence):
     def __init__(self):
         super().__init__()
         self._store = collections.OrderedDict()
@@ -167,12 +166,6 @@ class View(collections.Sequence):
     def store_count(self):
         return len(self._store)
 
-    def inbounds(self, index: int) -> bool:
-        """
-            Is this 0 <= index < len(self)
-        """
-        return 0 <= index < len(self)
-
     def _rev(self, idx: int) -> int:
         """
             Reverses an index, if needed
@@ -220,25 +213,51 @@ class View(collections.Sequence):
                 self._base_add(i)
         self.sig_view_refresh.send(self)
 
-    # API
+    """ View API """
+
+    # Focus
+    @command.command("view.focus.go")
+    def go(self, offset: int) -> None:
+        """
+            Go to a specified offset. Positive offests are from the beginning of
+            the view, negative from the end of the view, so that 0 is the first
+            flow, -1 is the last flow.
+        """
+        if len(self) == 0:
+            return
+        if offset < 0:
+            offset = len(self) + offset
+        if offset < 0:
+            offset = 0
+        if offset > len(self) - 1:
+            offset = len(self) - 1
+        self.focus.flow = self[offset]
+
     @command.command("view.focus.next")
     def focus_next(self) -> None:
         """
             Set focus to the next flow.
         """
-        idx = self.focus.index + 1
-        if self.inbounds(idx):
-            self.focus.flow = self[idx]
+        if self.focus.index is not None:
+            idx = self.focus.index + 1
+            if self.inbounds(idx):
+                self.focus.flow = self[idx]
+        else:
+            pass
 
     @command.command("view.focus.prev")
     def focus_prev(self) -> None:
         """
             Set focus to the previous flow.
         """
-        idx = self.focus.index - 1
-        if self.inbounds(idx):
-            self.focus.flow = self[idx]
+        if self.focus.index is not None:
+            idx = self.focus.index - 1
+            if self.inbounds(idx):
+                self.focus.flow = self[idx]
+        else:
+            pass
 
+    # Order
     @command.command("view.order.options")
     def order_options(self) -> typing.Sequence[str]:
         """
@@ -246,34 +265,58 @@ class View(collections.Sequence):
         """
         return list(sorted(self.orders.keys()))
 
-    @command.command("view.marked.toggle")
-    def toggle_marked(self) -> None:
-        """
-            Toggle whether to show marked views only.
-        """
-        self.show_marked = not self.show_marked
-        self._refilter()
-
-    def set_reversed(self, value: bool):
-        self.order_reversed = value
+    @command.command("view.order.reverse")
+    def set_reversed(self, boolean: bool) -> None:
+        self.order_reversed = boolean
         self.sig_view_refresh.send(self)
 
-    def set_order(self, order_key: typing.Callable):
+    @command.command("view.order.set")
+    def set_order(self, order_key: str) -> None:
         """
             Sets the current view order.
         """
+        if order_key not in self.orders:
+            raise exceptions.CommandError(
+                "Unknown flow order: %s" % order_key
+            )
+        order_key = self.orders[order_key]
         self.order_key = order_key
         newview = sortedcontainers.SortedListWithKey(key=order_key)
         newview.update(self._view)
         self._view = newview
 
-    def set_filter(self, flt: typing.Optional[flowfilter.TFilter]):
+    @command.command("view.order")
+    def get_order(self) -> str:
+        """
+        Returns the current view order.
+        """
+        order = ""
+        for k in self.orders.keys():
+            if self.order_key == self.orders[k]:
+                order = k
+        return order
+
+    # Filter
+    @command.command("view.filter.set")
+    def set_filter_cmd(self, filter_expr: str) -> None:
         """
             Sets the current view filter.
         """
+        filt = None
+        if filter_expr:
+            filt = flowfilter.parse(filter_expr)
+            if not filt:
+                raise exceptions.CommandError(
+                    "Invalid interception filter: %s" % filter_expr
+                )
+        self.set_filter(filt)
+
+    def set_filter(self, flt: typing.Optional[flowfilter.TFilter]):
         self.filter = flt or matchall
         self._refilter()
 
+    # View Updates
+    @command.command("view.clear")
     def clear(self) -> None:
         """
             Clears both the store and view.
@@ -283,7 +326,8 @@ class View(collections.Sequence):
         self.sig_view_refresh.send(self)
         self.sig_store_refresh.send(self)
 
-    def clear_not_marked(self):
+    @command.command("view.clear_unmarked")
+    def clear_not_marked(self) -> None:
         """
             Clears only the unmarked flows.
         """
@@ -294,35 +338,15 @@ class View(collections.Sequence):
         self._refilter()
         self.sig_store_refresh.send(self)
 
-    def add(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
-        """
-            Adds a flow to the state. If the flow already exists, it is
-            ignored.
-        """
-        for f in flows:
-            if f.id not in self._store:
-                self._store[f.id] = f
-                if self.filter(f):
-                    self._base_add(f)
-                    if self.focus_follow:
-                        self.focus.flow = f
-                    self.sig_view_add.send(self, flow=f)
-
-    def get_by_id(self, flow_id: str) -> typing.Optional[mitmproxy.flow.Flow]:
-        """
-        Get flow with the given id from the store.
-        Returns None if the flow is not found.
-        """
-        return self._store.get(flow_id)
-
-    @command.command("view.getval")
-    def getvalue(self, f: mitmproxy.flow.Flow, key: str, default: str) -> str:
+    # View Settings
+    @command.command("view.settings.getval")
+    def getvalue(self, flow: mitmproxy.flow.Flow, key: str, default: str) -> str:
         """
             Get a value from the settings store for the specified flow.
         """
-        return self.settings[f].get(key, default)
+        return self.settings[flow].get(key, default)
 
-    @command.command("view.setval.toggle")
+    @command.command("view.settings.setval.toggle")
     def setvalue_toggle(
         self,
         flows: typing.Sequence[mitmproxy.flow.Flow],
@@ -339,7 +363,7 @@ class View(collections.Sequence):
             updated.append(f)
         ctx.master.addons.trigger("update", updated)
 
-    @command.command("view.setval")
+    @command.command("view.settings.setval")
     def setvalue(
         self,
         flows: typing.Sequence[mitmproxy.flow.Flow],
@@ -354,42 +378,8 @@ class View(collections.Sequence):
             updated.append(f)
         ctx.master.addons.trigger("update", updated)
 
-    @command.command("view.load")
-    def load_file(self, path: mitmproxy.types.Path) -> None:
-        """
-            Load flows into the view, without processing them with addons.
-        """
-        spath = os.path.expanduser(path)
-        try:
-            with open(spath, "rb") as f:
-                for i in io.FlowReader(f).stream():
-                    # Do this to get a new ID, so we can load the same file N times and
-                    # get new flows each time. It would be more efficient to just have a
-                    # .newid() method or something.
-                    self.add([i.copy()])
-        except IOError as e:
-            ctx.log.error(e.strerror)
-        except exceptions.FlowReadException as e:
-            ctx.log.error(str(e))
-
-    @command.command("view.go")
-    def go(self, dst: int) -> None:
-        """
-            Go to a specified offset. Positive offests are from the beginning of
-            the view, negative from the end of the view, so that 0 is the first
-            flow, -1 is the last flow.
-        """
-        if len(self) == 0:
-            return
-        if dst < 0:
-            dst = len(self) + dst
-        if dst < 0:
-            dst = 0
-        if dst > len(self) - 1:
-            dst = len(self) - 1
-        self.focus.flow = self[dst]
-
-    @command.command("view.duplicate")
+    # Flows
+    @command.command("view.flows.duplicate")
     def duplicate(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
         """
             Duplicates the specified flows, and sets the focus to the first
@@ -401,7 +391,7 @@ class View(collections.Sequence):
             self.focus.flow = dups[0]
             ctx.log.alert("Duplicated %s flows" % len(dups))
 
-    @command.command("view.remove")
+    @command.command("view.flows.remove")
     def remove(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
         """
             Removes the flow from the underlying store and the view.
@@ -421,30 +411,30 @@ class View(collections.Sequence):
         if len(flows) > 1:
             ctx.log.alert("Removed %s flows" % len(flows))
 
-    @command.command("view.resolve")
-    def resolve(self, spec: str) -> typing.Sequence[mitmproxy.flow.Flow]:
+    @command.command("view.flows.resolve")
+    def resolve(self, flow_spec: str) -> typing.Sequence[mitmproxy.flow.Flow]:
         """
             Resolve a flow list specification to an actual list of flows.
         """
-        if spec == "@all":
+        if flow_spec == "@all":
             return [i for i in self._store.values()]
-        if spec == "@focus":
+        if flow_spec == "@focus":
             return [self.focus.flow] if self.focus.flow else []
-        elif spec == "@shown":
+        elif flow_spec == "@shown":
             return [i for i in self]
-        elif spec == "@hidden":
+        elif flow_spec == "@hidden":
             return [i for i in self._store.values() if i not in self._view]
-        elif spec == "@marked":
+        elif flow_spec == "@marked":
             return [i for i in self._store.values() if i.marked]
-        elif spec == "@unmarked":
+        elif flow_spec == "@unmarked":
             return [i for i in self._store.values() if not i.marked]
         else:
-            filt = flowfilter.parse(spec)
+            filt = flowfilter.parse(flow_spec)
             if not filt:
-                raise exceptions.CommandError("Invalid flow filter: %s" % spec)
+                raise exceptions.CommandError("Invalid flow filter: %s" % flow_spec)
             return [i for i in self._store.values() if filt(i)]
 
-    @command.command("view.create")
+    @command.command("view.flows.create")
     def create(self, method: str, url: str) -> None:
         try:
             req = http.HTTPRequest.make(method.upper(), url)
@@ -457,6 +447,74 @@ class View(collections.Sequence):
         f.request.headers["Host"] = req.host
         self.add([f])
 
+    @command.command("view.flows.load")
+    def load_file(self, path: mitmproxy.types.Path) -> None:
+        """
+            Load flows into the view, without processing them with addons.
+        """
+        try:
+            with open(path, "rb") as f:
+                for i in io.FlowReader(f).stream():
+                    # Do this to get a new ID, so we can load the same file N times and
+                    # get new flows each time. It would be more efficient to just have a
+                    # .newid() method or something.
+                    self.add([i.copy()])
+        except IOError as e:
+            ctx.log.error(e.strerror)
+        except exceptions.FlowReadException as e:
+            ctx.log.error(str(e))
+
+    def add(self, flows: typing.Sequence[mitmproxy.flow.Flow]) -> None:
+        """
+            Adds a flow to the state. If the flow already exists, it is
+            ignored.
+        """
+        for f in flows:
+            if f.id not in self._store:
+                self._store[f.id] = f
+                if self.filter(f):
+                    self._base_add(f)
+                    if self.focus_follow:
+                        self.focus.flow = f
+                    self.sig_view_add.send(self, flow=f)
+
+    def get_by_id(self, flow_id: str) -> typing.Optional[mitmproxy.flow.Flow]:
+        """
+            Get flow with the given id from the store.
+            Returns None if the flow is not found.
+        """
+        return self._store.get(flow_id)
+
+    # View Properties
+    @command.command("view.properties.length")
+    def get_length(self) -> int:
+        """
+            Returns view length.
+        """
+        return len(self)
+
+    @command.command("view.properties.marked")
+    def get_marked(self) -> bool:
+        """
+            Returns true if view is in marked mode.
+        """
+        return self.show_marked
+
+    @command.command("view.properties.marked.toggle")
+    def toggle_marked(self) -> None:
+        """
+            Toggle whether to show marked views only.
+        """
+        self.show_marked = not self.show_marked
+        self._refilter()
+
+    @command.command("view.properties.inbounds")
+    def inbounds(self, index: int) -> bool:
+        """
+            Is this 0 <= index < len(self)?
+        """
+        return 0 <= index < len(self)
+
     # Event handlers
     def configure(self, updated):
         if "view_filter" in updated:
@@ -473,7 +531,7 @@ class View(collections.Sequence):
                 raise exceptions.OptionsError(
                     "Unknown flow order: %s" % ctx.options.view_order
                 )
-            self.set_order(self.orders[ctx.options.view_order])
+            self.set_order(ctx.options.view_order)
         if "view_order_reversed" in updated:
             self.set_reversed(ctx.options.view_order_reversed)
         if "console_focus_follow" in updated:
@@ -532,7 +590,7 @@ class Focus:
     """
     def __init__(self, v: View) -> None:
         self.view = v
-        self._flow = None  # type: mitmproxy.flow.Flow
+        self._flow: typing.Optional[mitmproxy.flow.Flow] = None
         self.sig_change = blinker.Signal()
         if len(self.view):
             self.flow = self.view[0]
@@ -586,10 +644,10 @@ class Focus:
             self.flow = flow
 
 
-class Settings(collections.Mapping):
+class Settings(collections.abc.Mapping):
     def __init__(self, view: View) -> None:
         self.view = view
-        self._values = {}  # type: typing.MutableMapping[str, typing.Dict]
+        self._values: typing.MutableMapping[str, typing.Dict] = {}
         view.sig_store_remove.connect(self._sig_store_remove)
         view.sig_store_refresh.connect(self._sig_store_refresh)
 
