@@ -15,9 +15,9 @@ def pretty_json(s: bytes) -> Optional[bytes]:
 
 class ViewJSON(base.View):
     name = "JSON"
-    prompt = ("json", "s")
     content_types = [
         "application/json",
+        "application/json-rpc",
         "application/vnd.api+json"
     ]
 
