@@ -1,4 +1,3 @@
-import mitmproxy.net.http.url
 from mitmproxy import exceptions
 
 
@@ -78,15 +77,7 @@ def _assemble_request_headers(request_data):
     Args:
         request_data (mitmproxy.net.http.request.RequestData)
     """
-    headers = request_data.headers
-    if "host" not in headers and request_data.scheme and request_data.host and request_data.port:
-        headers = headers.copy()
-        headers["host"] = mitmproxy.net.http.url.hostport(
-            request_data.scheme,
-            request_data.host,
-            request_data.port
-        )
-    return bytes(headers)
+    return bytes(request_data.headers)
 
 
 def _assemble_response_line(response_data):
