@@ -13,6 +13,7 @@ import typing
 
 import click
 import collections
+import collections.abc
 import pydivert
 import pydivert.consts
 
@@ -58,7 +59,7 @@ class Resolver:
 
     def original_addr(self, csock: socket.socket):
         ip, port = csock.getpeername()[:2]
-        ip = re.sub("^::ffff:(?=\d+.\d+.\d+.\d+$)", "", ip)
+        ip = re.sub(r"^::ffff:(?=\d+.\d+.\d+.\d+$)", "", ip)
         ip = ip.split("%", 1)[0]
         with self.lock:
             try:
@@ -171,7 +172,7 @@ def MIB_TCPTABLE_OWNER_PID(size):
 TCP_TABLE_OWNER_PID_CONNECTIONS = 4
 
 
-class TcpConnectionTable(collections.Mapping):
+class TcpConnectionTable(collections.abc.Mapping):
     DEFAULT_TABLE_SIZE = 4096
 
     def __init__(self):
