@@ -126,7 +126,7 @@ class FlowDetails(tabs.Tabs):
             self.master.log.debug(error)
         # Give hint that you have to tab for the response.
         if description == "No content" and isinstance(message, http.HTTPRequest):
-            description = "No request content (press tab to view response)"
+            description = "No request content"
 
         # If the users has a wide terminal, he gets fewer lines; this should not be an issue.
         chars_per_line = 80
